@@ -2,12 +2,10 @@
 # this repository contains the full copyright notices and license terms.
 
 from trytond.pool import Pool
-from . import account
-from . import commission
-from . import invoice
-from . import sale
-from . import product
-from . import party
+
+from . import (
+    account, commission, commission_reporting, invoice, ir, party, product,
+    sale)
 
 
 def register():
@@ -18,13 +16,18 @@ def register():
         commission.AgentSelection,
         commission.Commission,
         commission.CreateInvoiceAsk,
+        commission_reporting.Agent,
+        commission_reporting.AgentTimeseries,
+        commission_reporting.Context,
         invoice.Invoice,
         invoice.InvoiceLine,
+        invoice.CreditInvoiceStart,
         product.Template,
         product.Template_Agent,
         product.Product,
         account.Journal,
         party.Party,
+        ir.EmailTemplate,
         module='commission', type_='model')
     Pool.register(
         sale.Sale,
@@ -33,6 +36,7 @@ def register():
         depends=['sale'])
     Pool.register(
         commission.CreateInvoice,
+        invoice.CreditInvoice,
         party.Replace,
         party.Erase,
         module='commission', type_='wizard')
