@@ -5,6 +5,7 @@ from decimal import Decimal
 from itertools import groupby
 
 from simpleeval import simple_eval
+
 try:
     from sql import Null
 except ImportError:
@@ -12,15 +13,15 @@ except ImportError:
 from sql.aggregate import Sum
 
 from trytond.i18n import gettext
-from trytond.model import ModelView, ModelSQL, MatchMixin, fields, \
-    sequence_ordered
-from trytond.pyson import Eval, Bool, If, Id, PYSONEncoder
-from trytond.tools import decistmt, grouped_slice, reduce_ids
-from trytond.pool import Pool
-from trytond.wizard import Wizard, StateView, StateAction, Button
-from trytond.transaction import Transaction
-
+from trytond.model import (
+    MatchMixin, ModelSQL, ModelView, fields, sequence_ordered)
+from trytond.modules.currency.fields import Monetary
 from trytond.modules.product import price_digits, round_price
+from trytond.pool import Pool
+from trytond.pyson import Bool, Eval, Id, If
+from trytond.tools import decistmt, grouped_slice, reduce_ids
+from trytond.transaction import Transaction
+from trytond.wizard import Button, StateAction, StateView, Wizard
 
 from .exceptions import FormulaError
 
@@ -32,7 +33,7 @@ class Agent(ModelSQL, ModelView):
         context={
             'company': Eval('company', -1),
             },
-        depends=['company'],
+        depends={'company'},
         help="The party for whom the commission is calculated.")
     type_ = fields.Selection([
             ('agent', 'Agent Of'),
@@ -153,7 +154,6 @@ class AgentSelection(sequence_ordered(), MatchMixin, ModelSQL, ModelView):
                 ('start_date', '<=', Eval('end_date')),
                 ()),
             ],
-        depends=['end_date'],
         help="The first date that the agent will be considered for selection.")
     end_date = fields.Date(
         "End Date",
@@ -162,26 +162,25 @@ class AgentSelection(sequence_ordered(), MatchMixin, ModelSQL, ModelView):
                 ('end_date', '>=', Eval('start_date')),
                 ()),
             ],
-        depends=['start_date'],
         help="The last date that the agent will be considered for selection.")
     party = fields.Many2One(
         'party.party', "Party", ondelete='CASCADE', select=True,
         context={
             'company': Eval('company', -1),
             },
-        depends=['company'])
+        depends={'company'})
     company = fields.Function(fields.Many2One('company.company', "Company"),
         'on_change_with_company', searcher='search_company')
     employee = fields.Many2One(
         'company.employee', "Employee", select=True,
         domain=[
             ('company', '=', Eval('company')),
-            ],
-        depends=['company'])
+            ])
 
     @classmethod
     def __setup__(cls):
         super().__setup__()
+        cls.__access__.add('agent')
         cls._order.insert(0, ('party', 'ASC NULLS LAST'))
         cls._order.insert(1, ('employee', 'ASC NULLS LAST'))
 
@@ -201,7 +200,8 @@ class AgentSelection(sequence_ordered(), MatchMixin, ModelSQL, ModelView):
         pattern = pattern.copy()
         if 'company' in pattern:
             pattern.pop('company')
-        date = pattern.pop('date', None) or Date.today()
+        with Transaction().set_context(company=self.company.id):
+            date = pattern.pop('date', None) or Date.today()
         if self.start_date and self.start_date > date:
             return False
         if self.end_date and self.end_date < date:
@@ -292,23 +292,25 @@ class PlanLines(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
         return 'amount'
 
     @classmethod
-    def validate(cls, lines):
-        super(PlanLines, cls).validate(lines)
+    def validate_fields(cls, lines, field_names):
+        super().validate_fields(lines, field_names)
+        cls.check_formula(lines, field_names)
+
+    @classmethod
+    def check_formula(cls, lines, field_names=None):
+        if field_names and 'formula' not in field_names:
+            return
         for line in lines:
-            line.check_formula()
-
-    def check_formula(self):
-        context = self.plan.get_context_formula(Decimal(0), None)
-
-        try:
-            if not isinstance(self.get_amount(**context), Decimal):
-                raise ValueError
-        except Exception as exception:
-            raise FormulaError(
-                gettext('commission.msg_plan_line_invalid_formula',
-                    formula=self.formula,
-                    line=self.rec_name,
-                    exception=exception)) from exception
+            context = line.plan.get_context_formula(Decimal(0), None)
+            try:
+                if not isinstance(line.get_amount(**context), Decimal):
+                    raise ValueError
+            except Exception as exception:
+                raise FormulaError(
+                    gettext('commission.msg_plan_line_invalid_formula',
+                        formula=line.formula,
+                        line=line.rec_name,
+                        exception=exception)) from exception
 
     def get_amount(self, **context):
         'Return amount (as Decimal)'
@@ -331,21 +333,23 @@ class Commission(ModelSQL, ModelView):
     _readonly_states = {
         'readonly': Bool(Eval('invoice_line')),
         }
-    _readonly_depends = ['invoice_line']
     origin = fields.Reference('Origin', selection='get_origin', select=True,
         readonly=True,
         help="The source of the commission.")
     date = fields.Date('Date', select=True, states=_readonly_states,
-        depends=_readonly_depends,
         help="When the commission is due.")
     agent = fields.Many2One('commission.agent', 'Agent', required=True,
-        states=_readonly_states, depends=_readonly_depends)
+        states=_readonly_states)
     product = fields.Many2One('product.product', 'Product', required=True,
-        states=_readonly_states, depends=_readonly_depends,
+        states=_readonly_states,
         help="The product that is used on the invoice line.")
-    amount = fields.Numeric('Amount', required=True, digits=price_digits,
+    base_amount = Monetary(
+        "Base Amount", currency='currency', digits='currency',
+        states=_readonly_states)
+    amount = Monetary(
+        "Amount", currency='currency', required=True, digits=price_digits,
         domain=[('amount', '!=', 0)],
-        states=_readonly_states, depends=_readonly_depends)
+        states=_readonly_states)
     currency = fields.Function(fields.Many2One('currency.currency',
             'Currency'), 'on_change_with_currency')
     type_ = fields.Function(fields.Selection([
@@ -353,7 +357,7 @@ class Commission(ModelSQL, ModelView):
                 ('out', 'Outgoing'),
                 ], 'Type'), 'on_change_with_type_')
     invoice_line = fields.Many2One('account.invoice.line', 'Invoice Line',
-        readonly=True, depends=['amount', 'type_'])
+        readonly=True)
     invoice_state = fields.Function(fields.Selection([
                 ('', ''),
                 ('invoiced', 'Invoiced'),
@@ -367,6 +371,7 @@ class Commission(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Commission, cls).__setup__()
+        cls.__access__.add('agent')
         cls._buttons.update({
                 'invoice': {
                     'invisible': Bool(Eval('invoice_line')),
@@ -428,6 +433,7 @@ class Commission(ModelSQL, ModelView):
     def invoice(cls, commissions):
         pool = Pool()
         Invoice = pool.get('account.invoice')
+        InvoiceLine = pool.get('account.invoice.line')
 
         def invoice_key(c):
             return c._group_to_invoice_key()
@@ -436,12 +442,12 @@ class Commission(ModelSQL, ModelView):
             return c._group_to_invoice_line_key()
         commissions.sort(key=invoice_key)
         invoices = []
-        to_write = []
+        invoice_lines = []
+        to_save = []
         for key, commissions in groupby(commissions, key=invoice_key):
             commissions = list(commissions)
             key = dict(key)
             invoice = cls._get_invoice(key)
-            invoice.save()
             invoices.append(invoice)
 
             commissions.sort(key=line_key)
@@ -449,13 +455,14 @@ class Commission(ModelSQL, ModelView):
                 commissions = [c for c in commissions if not c.invoice_line]
                 key = dict(key)
                 invoice_line = cls._get_invoice_line(key, invoice, commissions)
-                invoice_line.save()
-                to_write.extend([commissions, {
-                            'invoice_line': invoice_line.id,
-                            }])
-        if to_write:
-            cls.write(*to_write)
+                invoice_lines.append(invoice_line)
+                for commission in commissions:
+                    commission.invoice_line = invoice_line
+                    to_save.append(commission)
+        Invoice.save(invoices)
+        InvoiceLine.save(invoice_lines)
         Invoice.update_taxes(invoices)
+        cls.save(to_save)
 
     def _group_to_invoice_key(self):
         direction = {
@@ -516,11 +523,12 @@ class Commission(ModelSQL, ModelView):
 
         invoice_line = InvoiceLine()
         invoice_line.invoice = invoice
-        invoice_line.type = 'line'
-        invoice_line.product = product
-        invoice_line.quantity = 1
-        invoice_line.company = invoice.company
         invoice_line.currency = invoice.currency
+        invoice_line.company = invoice.company
+        invoice_line.type = 'line'
+        # Use product.id to instantiate it with the correct context
+        invoice_line.product = product.id
+        invoice_line.quantity = 1
 
         invoice_line.on_change_product()
 
@@ -549,6 +557,9 @@ class CreateInvoice(Wizard):
             domain.append(('agent.type_', '=', 'principal'))
         elif self.ask.type_ == 'out':
             domain.append(('agent.type_', '=', 'agent'))
+        if self.ask.agents:
+            agents = [agent.id for agent in self.ask.agents]
+            domain.append(('agent', 'in', agents))
         return domain
 
     def do_create_(self, action):
@@ -560,11 +571,7 @@ class CreateInvoice(Wizard):
         commissions = Commission.browse(commissions)
         Commission.invoice(commissions)
         invoice_ids = list({c.invoice_line.invoice.id for c in commissions})
-        encoder = PYSONEncoder()
-        action['pyson_domain'] = encoder.encode(
-            [('id', 'in', invoice_ids)])
-        action['pyson_search_value'] = encoder.encode([])
-        return action, {}
+        return action, {'res_id': invoice_ids}
 
 
 class CreateInvoiceAsk(ModelView):
@@ -575,14 +582,12 @@ class CreateInvoiceAsk(ModelView):
             If(Eval('to') & Eval('from_'), [('from_', '<=', Eval('to'))],
                 []),
             ],
-        depends=['to'],
         help="Limit to commissions from this date.")
     to = fields.Date('To',
         domain=[
             If(Eval('from_') & Eval('to'), [('to', '>=', Eval('from_'))],
                 []),
             ],
-        depends=['from_'],
         help="Limit to commissions to this date.")
     type_ = fields.Selection([
             ('in', 'Incoming'),
@@ -590,6 +595,16 @@ class CreateInvoiceAsk(ModelView):
             ('both', 'Both'),
             ], 'Type',
         help="Limit to commissions of this type.")
+    agents = fields.Many2Many(
+        'commission.agent', None, None, "Agents",
+        domain=[
+            If(Eval('type_') == 'in',
+                ('type_', '=', 'principal'), ()),
+            If(Eval('type_') == 'out',
+                ('type_', '=', 'agent'), ()),
+        ],
+        help="Limit to commissions for these agents.\n"
+        "If empty all agents of the selected type are used.")
 
     @staticmethod
     def default_type_():
